#!/usr/bin/lua5.1
-- -*- Lua -*-
-- gitano-smart-http
--
-- Git (with) Augmented network operations -- User authentication wrapper
--
-- Copyright 2014 Codethink Ltd
--
--

-- Gitano modules installed into /usr/share/lua/5.1/?.lua

local gitano = require "gitano"
local gall = require "gall"
local luxio = require "luxio"
local sio = require "luxio.simple"

gitano.config.lib_bin_path("/usr/lib/gitano/bin")
gitano.config.share_path("/usr/share/gitano")
gitano.i18n.set_langpack_path("/usr/share/gitano/lang") gitano.i18n.set_category()
gitano.plugins.load_plugins {"/etc/gitano/plugins", "/usr/lib/gitano/plugins"}

local stdout = sio.stdout

function parse_get_request()
   query_string = os.getenv("QUERY_STRING")

   if query_string then
      command = string.gsub(query_string, "^service=", "")
      repo = string.match(os.getenv("PATH_INFO"), '/(.+)/info/refs')
      return command .. " '" .. repo .. "'"
   end

   return nil
end

function parse_post_request()
   path_info = os.getenv("PATH_INFO")

   if path_info then
      repo, command = string.match(path_info, "/(.+)/(.+)")
      return command .. " '" .. repo .. "'"
   end

   return nil
end

function parse_request(request_method)
   if request_method == "GET" then
      return parse_get_request()
   elseif request_method == "POST" then
      return parse_post_request()
   end
end

request_method = os.getenv("REQUEST_METHOD")

if request_method == "GET" or request_method == "POST" then
   local user = os.getenv("REMOTE_USER") or "gitano/anonymous"
   local cmdline = parse_request(request_method)

   local start_log_level = gitano.log.get_level()
   gitano.log.cap_level(gitano.log.level.INFO)
   local transactionid = gitano.log.syslog.open()

   local authorized, cmd, parsed_cmdline, config, env, repo
   if cmdline then
      authorized, cmd, parsed_cmdline, config, env, repo =
         gitano.auth.is_authorized(user, "http", cmdline,
                                   os.getenv("GITANO_ROOT"),
                                   transactionid,
                                   start_log_level)
   end
   if cmdline and authorized then
      local what, why

      what, why = repo:git_command{"http-backend"}

      if what ~= "exit" then
         stdout:write("Status: 500 Internal Server Error\r\n\r\n")
      end
   else
      stdout:write("Status: 403 Forbidden\r\n\r\n")
   end

   gitano.log.syslog.close()
else
   stdout:write("Status: 405 Method Not Allowed\r\n")
   stdout:write("Allow: GET, POST\r\n\r\n")
end
